MSP430 Camera Node
Copyright (C) 2007 Joseph Bosman, Matthew Conway, Phong Dam, Steve Olivieri,
	Ipek Ozil, Janelle Tavares, Brandon Steacy

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA


#include <msp430x16x.h>
#include <string.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include "mmc.h"

#ifndef NODE_H
#define NODE_H

/* How many pictures to take when the PIR is triggered */
const unsigned short int num_pic_taken = 2;

/* Image resolution, can be:
 * 	0x03 : 160x128
 * 	0x05 : 320x240
 * 	0x07 : 640x480
 */
const unsigned char resolution = 0x05;

/* Interrupt flag for timer A */
unsigned short int timeflag_a = 0;

/* Interrupt flag for timer B */
unsigned short int timeflag_b = 0;

/* How many 1/10 seconds we've waited */
unsigned short int tsec = 0; 

/* Camera command buffers */
unsigned char cmdSYNC[6];
unsigned char cmdACK[6];
unsigned char cmdReset[6];
unsigned char cmdPowerOff[6];
unsigned char cmdInitial[6];
unsigned char cmdSetPackageSize[6];
unsigned char cmdSetBaudRate[6];
unsigned char cmdSnapShot[6];
unsigned char cmdGetPic[6];
unsigned char cmdData[6];
unsigned char cmdEndPic[6];

/* Size of a camera command */
const unsigned short int cmdSize = 6;

/* Size of a double camera command */
const unsigned short int cmdACKsize = 12;

/* Size of an image data packet */
const unsigned short int packageSize = 262;

/* TODO: Phong */
const unsigned short int maxSendSYNC = 60;

/* Temporary buffer to store data packets from the camera */
unsigned char dummybuffer[262];

/* TODO: Phong */
unsigned short int dummyLen = 0;

/* TODO: Phong */
unsigned short int dummyRecv = 0;

/* Buffer for receiving data from the camera */
static unsigned char recvbuffer[262];

/* Which half of the mmc_buffer to store the packet in, can be:
 * 	0 : first half (256 bytes)
 * 	1 : second half (256 bytes)
 */
unsigned short int bufFlag = 0;

/* Camera flags (TODO: Phong) */
unsigned short int numberOfpackage;
unsigned short int lastPackDataSize;
unsigned short int currentLen = 0;
unsigned short int finishSYNCflag = 0;
unsigned short int prepSYNCsuccess = 0;
unsigned short int initialACKflag = 0;
unsigned short int setPackACKflag = 0;
unsigned short int snapACKflag = 0;
unsigned short int getPicACKflag = 0;
unsigned short int dataPackflag = 0;

/* Signals the MSP430 that data is ready. */
unsigned short int dataReady = 0;

/* Duration of the SD card initialization */
const unsigned short int timeout = 30;

/* Flag for SD initialization timeout */
unsigned short int timeoutFlag = 0;

/* How many blocks we've written to the SD card (resetting this variable
 * will cause us to overwrite any existing data)
 */
unsigned short int block = 0;

/* State of the SD card, can be:
 * 	0 : not found
 * 	1 : found
 */
extern unsigned char card_state;
unsigned char card_state = 0;

/* Buffer for holding data prior to transferring it to the SD card */
extern unsigned char mmc_buffer[512];

/* Size of each image */
unsigned int picSize[100];

/* Number of pictures taken */
unsigned short int numOfPic = 0;

/* Number of memory blocks read */
unsigned int numBlocks;

/* nHOST_INT interrupt boolean for the EM260 */
volatile unsigned char nHOST_INT;

/* The EM60's node ID */
unsigned char nodeID[2];

/* The sequence number for EM260 EZSP frames */
volatile unsigned char sq;

/* The sequence number for EM260 unicast messages */
volatile unsigned char sq2;

/* Interrupt functions */
#ifdef __IAR_SYSTEMS_ICC__
  #pragma vector=PORT1_VECTOR
    __interrupt void em260(void);
  #pragma vector=PORT2_VECTOR
    __interrupt void port_2(void);
  #pragma vector=TIMERA0_VECTOR
    __interrupt void Timer_A0(void);
  #pragma vector=TIMERB0_VECTOR
    __interrupt void Timer_B0(void);
  #pragma vector=UART0RX_VECTOR
    __interrupt void usart0_rx(void);
#else
  interrupt (PORT1_VECTOR) em260(void);
  interrupt (PORT2_VECTOR) port_2(void);
  interrupt (TIMERA0_VECTOR) Timer_A0(void);
  interrupt (TIMERB0_VECTOR) Timer_B0(void);
  interrupt (UART0RX_VECTOR) usart0_rx(void);
#endif

/* Starts running timer A. */
void runtimera(void);

/* Stops running timer A. */
void stoptimera(void);

/* Starts running timer B. */
void runtimerb(void);

/* Stops running timer B. */
void stoptimerb(void);

/* Initializes the MSP430. */
void init_sys(void);

/* Resets the camera variables and arrays. */
void reset_sys(void);

/* Initializes the SD card. */
void init_SD(void);

/* Enables SPI on USART0 of the MSP430. */
void enableSPI0(void);

/* Disables SPI on USART0 of the MSP430. */
void disableSPI0(void);

/* Enables UART on USART0 of the MSP430. */
void enableUART0(void);

/* Disables UART on USART0 of the MSP430. */
void disableUART0(void);

/* Waits for 1/10 of a second. */
void wait(void);

/* Waits for 1/100 of a second. */
void wait_a(void);

/* Waits for (s * 1/100) seconds.  For example, if s is equal to 10, this
 * function will wait for 0.10 seconds.
 * 	@param s: the number of 1/100 seconds to wait
 */
void wait_a_sec(unsigned short int s);

/* Waits for (s * 1/10) seconds.  For example, if s is equal to 10, this
 * function will wait for 1.00 seconds.
 */
void waitsec(unsigned short int s);

/* Turns on the MSP430 LED. */
void LEDOn(void);

/* Turns off the MSP430 LED. */
void LEDOff(void);

/* Turns on LED 1. */
void LED1On(void);

/* Turns off LED 1. */
void LED1Off(void);

/* Turns on LED 2. */
void LED2On(void);

/* Turns off LED 2. */
void LED2Off(void);

/* Turns on LED 3. */
void LED3On(void);

/* Turns off LED 3. */
void LED3Off(void);

/* Turns on LED 4. */
void LED4On(void);

/* Turns off LED 4. */
void LED4Off(void);

/* Blinks the specified LED.
 * 	@param i: the LED to blink
 */
void blinkLED(unsigned short int i);

/* Configures the camera command buffers. */
void config_camera_cmd(void);

/* Sends a command to the camera.
 * 	@param c: the command to send
 */
void send_cmd(unsigned char c[]);

/* Receives an ACK command from the camera. */
void recv_cmd(void);

/* Sends a sequence of SYNC commands to the camera. */
void sendSYNC(void);

/* Checks to see if the command received from the camera is NACK. */
void check(void);

/* Initializes the camera system.  This involves the following tasks:
 * 	- Sending the "initialize" command to the camera
 * 	- Sending the "set package size" command to the camera
 */
void cam_prep(void);

/* Takes a picture with the camera.  This involves the following tasks:
 * 	- Sending the "snap shot" command to the camera
 * 	- Sending the "get picture" command to the camera
 */
void take_pic(void);

/* Gets an image from the camera. */
void get_image(void);

/* Gets a specific data packet from the camera.
 * 	@param id: the data packet to get
 */
void get_package(unsigned char id);

/* Calculates the number of packets for the last image taken. */
void cal_numbOfpack(void);

/* Calculates the size of the last image packet. */
void cal_lastPackSize(void);

/* Transfers image data packets to the SD card. */
void transfer_pack(void);

/* Prepares the mmc_buffer for writing data to the SD card. */
void prep_buffer(void);

/* Prepares the mmc_buffer with the last packet (TODO: Phong). */
void prep_last_buffer(void);

/* Clears the mmc_buffer. */
void clear_mmc_buffer(void);

/* Converts a character to an integer */
unsigned short int hex2int(unsigned char c1 ,unsigned char c2);

/* Calculates how many packets are going to be sent to the base
 * station by getting the total numer of bytes and sends the
 * "IMAGE_READY" signal to the base station specifying number
 * of packets. After it receives an "IMAGE_READY_OK" from 
 * the base station, it reads from the memory and sends 64
 * bytes at a time until a whole image is sent.
 *	@param imgNum: the number of the image taken
 */
void transmitPicture(unsigned int imgNum);

/* Initializes the EM260 by performing the following tasks:
 *      - resets the EM260
 *      - performs a reset transaction over SPI
 *      - performs a "verion" transaction over SPI
 *      - performs a "SPI status" transaction over SPI
 */
unsigned int initEM260(void);

/* Sends a string to the EM260 over SPI1.
 *      @param buff: a pointer to the string to send
 *      @param len: the length of the string
 */
void toEM260(unsigned char* buff, unsigned int len);

/* Receives data from the EM260 over SPI1.
 *      @param buff: a pointer to the buffer to store the response in
 *      @return: the number of bytes received (and stored in buff)
 */
unsigned int fromEM260(unsigned char* buff);


/* Sets the configuration values for the EM260.
 * 	@param params: the configuration to set and its new value
 * 	@return: result of the command
 */
unsigned char EM260_setConfigurationValue(unsigned char* params);

/* Issues a callback() command to the EM260 and receives a response.
 * Typically, callbacks are made after calling other commands.  In the
 * case that the EM260 must send us a message asynchronously, the
 * nHOST_INT interrupt will be triggered and a callback must be made.
 *      @param buff: pointer to the buffer to store the respnse in    
 */
void EM260_callback(unsigned char* buff);

/* Calls the EZSP networkInit() function.  The result of calling
 * this function depends on whether or not the EM260 was joined
 * to a network on last power off.  If so, a callbac must follow
 * this command.  In addition, the EM260 will once again be on
 * that network.  If not, no callback should be issued and the
 * EM260 will not be joined to any networks.
 *      @return: the network status
 */
void EM260_networkInit();

/* Calls the EZSP leaveNetwork() function.  The result of this
 * call is the return value for EM260_leaveNetwork().  If this
 * call is successful, the EM260 will leave any network that it
 * is joined to.  Else, the state of the network is unknown.
 *      @return: the status of the command
 */
unsigned char EM260_leaveNetwork();

/* Calls the EZSP networkState() function. The result of this
 * call is the return value for EM260_networkState() which 
 * tells the state the network is in.
 *	@return: status of the network
 */
unsigned char EM260_networkState ();

/* Calls the EZSP startScan() function. The result of this 
 * call is the return value for EM260_scanNetwork(). If this
 * call is successful, there will be a callback to the function
 * that either says scan is complete or network found.
 *	@return: result of the command
 */
unsigned char EM260_scanNetwork ();

/* Calls the EZSP joinNetwork() function. The result of this
 * call is the return value for EM260_justJoinNetwork(). If
 * this call is successful, EM260 will join the found network.
 * There will also be callback that says network is up.
 *	@param params: parameters from the callback of 
 *	scanNetwork() that include the PAN ID, short PAN ID 
 *	and the channel.
 *	@retur: result of the command
 */
unsigned char EM260_justJoinNetwork(unsigned char* params);

/* Calls the EZSP getNodeID() function. The result of this
 * call is the return values for EM260_getNodeID(). If this 
 * call is successful, it will get the Node ID of the device.
 *	@param: pointer to a buffer to store the result of 
 *	the call.
 */
 
void EM260_getNodeID(unsigned char* buff);

/* Sends a unicast message to another node on the network via the
 * EZSP sendUnicast() function.  If this function is successful,
 * a callback must follow.  This function always uses ProfileId 0,
 * ClusterId 0, and EndPoint 0x01 (source and destination).  Messages
 * are always tagged with 0x64, route discovery is enabled, retries
 * are enabled, and the source EUI64 is always sent before the message.
 *      @param addr: the two-byte NodeID to send the message to
 *      @param message: the message to send
 *      @param len: the length of message
 */

unsigned char EM260_sendUnicast(unsigned char* message, unsigned char* nodeID,
	unsigned char len);

/* Calls the EM260_networkState(), EM260_leaveNetwork(), EM260_networkScan()
 * and EM260_justJoinNetwork() functions. First, gets the state of the network
 * by calling EM260_networkState(). If the result says that the device is not
 * a part of a network, it calls EM260_scanNetwork()to find any available networks 
 * and then calls EM260_justJoinNetwork(). If the result of EM260_networkState()
 * is joined to a network, it first calls EM260_leaveNetwork() to leave the 
 * current network and then goes through the joinin procedure.
 */
void joinNetwork();

#endif /* NODE_H */
