PC Base Station
Copyright (C) 2007 Joseph Bosman, Steve Olivieri, Ipek Ozil, Brandon Steacy

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA


#include "em250.h"

void initTimeStruct(LPSYSTEMTIME t)
{
	/* Set the date to 00/00/0000, 00:00:00. */
	t->wYear = 0;
	t->wMonth = 0;
	t->wDay = 0;
	t->wHour = 0;
	t->wMinute = 0;
	t->wSecond = 0;
	
	return;
}

void setEqualTimeStructs(LPSYSTEMTIME t1, LPSYSTEMTIME t2)
{
	/* Set t1 equal to t2 to second precision. */
	t1->wYear = t2->wYear;
	t1->wMonth = t2->wMonth;
	t1->wDay = t2->wDay;
	t1->wHour = t2->wHour;
	t1->wMinute = t2->wMinute;
	t1->wSecond = t2->wSecond;
	
	return;
}

int compareTimeStructs(LPSYSTEMTIME t1, LPSYSTEMTIME t2)
{
	/* Compare the two dates from the largest unit to the smallest.
	 * If at any time one is larger than the other, stop comparing.
	 */
	if(t1->wYear > t2->wYear)
		return GREATER_THAN;
	if(t2->wYear > t1->wYear)
		return LESS_THAN;
		
	if(t1->wMonth > t2->wMonth)
		return GREATER_THAN;
	if(t2->wMonth > t1->wMonth)
		return LESS_THAN;
		
	if(t1->wDay > t2->wDay)
		return GREATER_THAN;
	if(t2->wDay > t1->wDay)
		return LESS_THAN;
		
	if(t1->wHour > t2->wHour)
		return GREATER_THAN;
	if(t2->wHour > t1->wHour)
		return LESS_THAN;
		
	if(t1->wMinute > t2->wMinute)
		return GREATER_THAN;
	if(t2->wMinute > t1->wMinute)
		return LESS_THAN;
		
	if(t1->wSecond > t2->wSecond)
		return GREATER_THAN;
	if(t2->wSecond > t1->wSecond)
		return LESS_THAN;
	
	/* The two dates are equal, to the precision of a second. */
	return EQUAL;
}

HANDLE setupCOM(LPCSTR port)
{
	/* Serial port configuration settings. */
	DCB comConfig;
	
	/* Serial port timeout settings. */
	COMMTIMEOUTS comTimeouts;
	
	/* The file handle to return to the caller. */
	HANDLE ret;
	
	/* Open the serial port. */
	ret = CreateFile((LPCSTR)port, GENERIC_READ|GENERIC_WRITE, 0, NULL,
		OPEN_EXISTING, 0, NULL);
	if(ret == INVALID_HANDLE_VALUE)
		return INVALID_HANDLE_VALUE;

	/* Get the port's configuration settings, then change them to better
	 * match the ETRX2USB device.
	 */
	if(GetCommState(ret, &comConfig))
	{
		#ifdef TELEGESIS
		  comConfig.BaudRate = 19200;
		#else
		  comConfig.BaudRate = 38400;
		#endif
		comConfig.ByteSize = 8;
		comConfig.Parity = NOPARITY;
		comConfig.StopBits = ONESTOPBIT;
		comConfig.fBinary = FALSE;
		comConfig.fParity = FALSE;
	}
	else
		return INVALID_HANDLE_VALUE;
	if(!SetCommState(ret, &comConfig))
		return INVALID_HANDLE_VALUE;

	/* Get the port's timeout settings, then set them all to 25 ms.  This
	 * is probably not an ideal solution.
	 */
	if(GetCommTimeouts(ret, &comTimeouts))
	{
		comTimeouts.ReadIntervalTimeout = 25;
		comTimeouts.ReadTotalTimeoutConstant = 25;
		comTimeouts.ReadTotalTimeoutMultiplier = 25;
		comTimeouts.WriteTotalTimeoutConstant = 25;
		comTimeouts.WriteTotalTimeoutMultiplier = 25;
	}
	else
		return INVALID_HANDLE_VALUE;
	if(!SetCommTimeouts(ret, &comTimeouts))
		return INVALID_HANDLE_VALUE;

	/* Empty the input and output buffers for the serial port. */
	PurgeComm(ret, PURGE_RXCLEAR|PURGE_TXCLEAR);
	
	/* Return the file handle for the serial port. */
	return ret;
}

int writeSerial(LPCSTR data, PHANDLE comPort, UINT len, BOOL command)
{
	/* Single-character buffer for writing strings to the serial port. */
	CHAR zBuff;
	
	/* How many bytes we wrote on the last write call. */
	DWORD bWritten;
	
	/* How many bytes we've read. */
	DWORD bRead;
	
	/* Loop variable. */
	UINT i;
	
	/* Write a command to the serial port. */
	for(i = 0; i < len; i++)
	{
		zBuff = *(data+i);
		if(!WriteFile(*comPort, (LPVOID)&zBuff, 1, &bWritten, NULL))
			return FAILURE;
	}
	
	/* Send a carriage return to finish the command. */
	if(command)
	{
		zBuff = '\r';
		if(!WriteFile(*comPort, (LPVOID)&zBuff, 1, &bRead, NULL))
			return FAILURE;
	}

	return SUCCESS;
}

int readMessage(PHANDLE in, LPSTR buff)
{
	/* Temporary buffer. */
	CHAR zBuff;
	
	/* Stores how many bytes the last ReadFile() read. */
	DWORD bRead;
	
	/* Stores how many total bytes we've read. */
	INT i;
	
	/* Initialize i.  Then, read in the data one byte at a
	 * time and write it to buff.  Data is read this way for COM
	 * support.  We finish when there is no more data to be read.
	 */
	i = 0;
	do
	{
		ReadFile(*in, (LPVOID)&zBuff, 1, &bRead, NULL);
		if(bRead > 0)
		{
			*(buff+i) = zBuff;
			++i;
		}
	} while (bRead > 0);
	*(buff+i) = '\0';

	return i;
}

LPSTR getMessage(PHANDLE in, LPCSTR message, PHANDLE out)
{
	/* Temporary buffer for storing data that we read from the COM port. */
	LPSTR buff;
	
	/* Temporary buffer that points to part of buff containing the desired
	 * message (if it exists) or nothing (otherwise).
	 */
	LPSTR tBuff;
	
	/* We copy tBuff into a separate buffer for returning to avoid memory
	 * leaks.
	 */
	LPSTR retBuff;
	
	/* How many bytes we wrote. */
	DWORD bWritten;
	
	/* How many bytes we read. */
	INT i;

	/* Since we're storing temporary data in buff, we need to allocate
	 * memory for it now.  Since we use tBuff as a BOOL, we leave it
	 * NULL for now and only give it a value when we find the message
	 * that we're looking for (or an ERROR).
	 */
	buff = NULL;
	buff = (LPSTR)calloc(150, sizeof(CHAR));
	tBuff = NULL;

	/* While we haven't found the desired message (or an ERROR), keep
	 * looking!
	 */
	while(!tBuff)
	{
		/* Read data from the COM port. */
		memset(buff, 0, 150);
		i = readMessage(in, buff);
		if(*(buff+i) != '\0')
			*(buff+i) = '\0';

		/* Search that data for our desired message. If we don't find it
		 * search for an ERROR message.
		 */
		tBuff = strstr(buff, message);
		if(!tBuff)
			tBuff = strstr(buff, "ERROR:");

		/* If out is not NULL, write any data that we've read to the file
		 * pointed to by out.  Usually, this is a log file.  And, we don't
		 * really care if it fails.
		 */
		if(out)
			WriteFile(*out, (LPVOID)buff, i, &bWritten, NULL);

	}

	/* Since we have something to return, let's allocate memory for retBuff
	 * and then copy the useful data into it.  The caller of getMessage()
	 * is responsible for freeing retBuff.
	 */
	retBuff = (LPSTR)strdup(tBuff);
	
	free(buff);
	free(tBuff);
	return retBuff;
}

int formatMessage(LPSTR tBuff, LPSTR buff, LPSTR hwID, INT binary, INT l_tBuff, INT l_buff, INT l_hwID)
{
	/* How many bytes are in the real message. */
	UINT i;
	i = 0;
	
	/* The message contains two important things.  The hardware ID number
	 * of the node that it originated from, and the actual data of the
	 * message.  Messages begin with two useless characters, followed by
	 * a message type (NEWNODE, UCAST, etc.) and then a colon.  Next, the
	 * hardware ID number (16 hex digits), an equals sign (=), and finally
	 * the message.  Note that sytem commands (such as NEWNODE) do not
	 * contain anything after the hardware ID number.  Some commands have
	 * a space (' ') between the colon and the message, while others
	 * do not.  At the end, we have the following:
	 * 	tBuff: the message
	 *	hwID: the hardware ID of the sender
	 *	buff: hwID=buff
	 */
	 
	/* Remove the leading and trailing characters from the initial message. */
	memset(tBuff, 0, l_tBuff);
	memcpy(tBuff, buff+2, l_buff-5);
	
	/* Copy everything after the colon into buff. */
	memset(buff, 0, l_buff);
	memcpy(buff, (LPSTR)memchr(tBuff, (INT)':', l_buff)+1, l_buff);
	
	/* If this command places a space between the colon and the message,
	 * remove it.
	 */
	if(memchr(buff, (INT)' ', 1))
	{
		memset(tBuff, 0, l_tBuff);
		memcpy(tBuff, buff+1, l_buff);
		memset(buff, 0, l_buff);
		memcpy(buff, tBuff, l_tBuff);
	}

	/* If this is a binary message, we need to figure out how many bytes to
	 * read.  This number is stored in the last two characters of the
	 * message.  Between this number and the actual message is a comma.
	 * Also, this is a convenient time to get the hardware ID.
	 */
	if(binary)
	{
		memset(hwID, 0, l_hwID);
		memcpy(hwID, buff+17, 2);
		i = strtol(hwID, NULL, 16);
		memset(hwID, 0, l_hwID);
		memcpy(hwID, buff, 16);
	}	
	else
	{
		/* Copy the hardware ID into hwID. */
		memset(hwID, 0, l_hwID);
		memcpy(hwID, buff, 16);
	}
	
	/* Finally, copy the message into tBuff. */
	if(binary)
	{
		memset(tBuff, 0, l_tBuff);
		memcpy(tBuff, buff+20, l_buff);
	}
	else
	{
		memset(tBuff, '\0', l_tBuff);
		memcpy(tBuff, buff+17, l_buff);
		tBuff[l_tBuff-1] = '\0';
	}
	
	/* For a binary string, return i (hex).  Else, return the length
	 * of tBuff (decimal).  Both should be the length of the actual
	 * message.
	 */
	return (binary ? i : strlen(tBuff));
}

int main(int argc, char* argv[])
{
	/* Handle for the log file we're writing to */
	HANDLE log;
	
	/* Handle for the file we're reading from. */
	HANDLE serial;
	
	/* Handle for the image file we're writing. */
	HANDLE imageFile;
	
	/* The COM port to connect to. */
	LPSTR comPort;
	
	/* Directory to store files in. */
	LPSTR fileDir;
	
	/* Filename for the output file. */
	LPSTR fileName;

	/* Temporary filename for moving files. */
	/*LPSTR tmpName;*/
	
	/* Buffers for storing text to be written to a file. */
	LPSTR buff;
	LPSTR tBuff;
	
	/* The hardware ID number for the sender. */
	LPSTR hwID;
	LPSTR hwID2;
	
	/* Storage for how many bytes we wrote with WriteFile(). */
	DWORD bWritten;
	
	/* Storage for how many bytes we read with ReadFile(). */
	DWORD bRead;

	/* Event Mask for COM port. */
	DWORD eventMask;
	
	/* The number of packets that we expect an image to be. */
	DWORD numPackets;
	
	/* The number of packets that we've receieved so far. */
	DWORD recPackets;
	
	/* Stores how many files we've made today. */
	INT filesMade;
	
	/* Stores the current local time. */
	SYSTEMTIME currTime;
	
	/* Stores the last local time used. */
	SYSTEMTIME lastTime;
	
	/* Used as a timer for timeouts. */
	SYSTEMTIME timer;
	
	/* How long the last image transfer took in milliseconds. */
	DWORD tSec;
	
	/* Loop variable. */
	INT i;
	
	/* Make sure that they specify the required command
	 * arguments (COM port and directory).
	 */
	if(argc < 5)
		return MAIN_INPUT_ERROR;
		
	/* Allocate memory for our strings. */
	comPort = (LPSTR)calloc(6, sizeof(CHAR));
	buff = (LPSTR)calloc(150, sizeof(CHAR));
	hwID = (LPSTR)calloc(17, sizeof(CHAR));
	hwID2 = (LPSTR)calloc(17, sizeof(CHAR));
	fileDir = (LPSTR)calloc(999, sizeof(CHAR));
	fileName = (LPSTR)calloc(1024, sizeof(CHAR));
	/*tmpName = (LPSTR)calloc(1024, sizeof(CHAR));*/
	tBuff = NULL;
	
	/* Initialize variables. */
	initTimeStruct(&currTime);
	initTimeStruct(&lastTime);
	initTimeStruct(&timer);
	filesMade = 0;
	bRead = 0;
	bWritten = 0;
	numPackets = 0x0;
	recPackets = 0x0;
	
	/* Parse input. */
	for(i = 1; i < argc; i++)
	{
		if(strcmp(argv[i], "-c") == 0)
			sprintf(comPort, "COM%s:", argv[++i]);
		else if(strcmp(argv[i], "-d") == 0)
			sprintf(fileDir, "%s", argv[++i]);
		else
			return MAIN_INPUT_ERROR;
	}

	/* Open the serial port. */
	serial = setupCOM((LPCSTR)comPort);
	if(serial == INVALID_HANDLE_VALUE)
	{
		fprintf(stderr, "Could not open %s.\n", comPort);
		free(comPort);
		free(buff);
		free(hwID);
		free(hwID2);
		free(fileDir);
		free(fileName);
		return MAIN_SERIAL_PORT_FAIL;
	}

	/* Create the log file to write to. */
	GetLocalTime((LPSYSTEMTIME)&currTime);
	sprintf(fileName, "%sLOG_%.2d%.2d%.2d_%.2d%.2d%.2d.txt", fileDir,
		currTime.wDay, currTime.wMonth, currTime.wYear,
		currTime.wHour, currTime.wMinute, currTime.wSecond);
	log = CreateFile((LPCSTR)fileName, GENERIC_WRITE, FILE_SHARE_READ,
		NULL, CREATE_ALWAYS, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if(log == INVALID_HANDLE_VALUE)
	{
		fprintf(stderr, "Failed to open the log file: %s.\n", fileName);
		CloseHandle((PVOID)&serial);
		free(comPort);
		free(buff);
		free(tBuff);
		free(hwID);
		free(hwID2);
		free(fileDir);
		free(fileName);
		return MAIN_LOG_FILE_FAIL;
	}

	/* Tell the EM250 to leave any PANs that it might be in. If we get an
	 * ERROR here, it's because we weren't already in a network.  We can
	 * ignore that error.
	 */
	if(writeSerial("AT+DASSL", &serial, strlen("AT+DASSL"), TRUE))
	{
		fprintf(stderr, "Could not write to %s.\n", comPort);
		CloseHandle((PVOID)&serial);
		CloseHandle((PVOID)&log);
		free(comPort);
		free(buff);
		free(hwID);
		free(hwID2);
		free(fileDir);
		free(fileName);
		return MAIN_DATA_WRITE_ERROR;
	}
	
	strncpy(buff, "LeftPAN", 8);
	tBuff = getMessage(&serial, buff, &log);
	free(tBuff);
	memset(buff, 0, 8);
	
	#ifdef TELEGESIS
	/* Now, we need to turn on local echo, because it might have been off. */
	if(writeSerial("ATS0B4=0", &serial, strlen("ATS0B4=0"), TRUE))
	{
		fprintf(stderr, "Could not write to %s.\n", comPort);
		CloseHandle((PVOID)&serial);
		CloseHandle((PVOID)&log);
		free(comPort);
		free(buff);
		free(hwID);
		free(hwID2);
		free(fileDir);
		free(fileName);
		return MAIN_DATA_WRITE_ERROR;
	}
	
	strncpy(buff, "OK", 3);
	tBuff = getMessage(&serial, "OK", &log);
	if(strstr(tBuff, "ERROR:"))
	{
		fprintf(stderr, "Could not set register value 0B4, %s.\n", tBuff);
		CloseHandle((PVOID)&serial);
		CloseHandle((PVOID)&log);
		free(comPort);
		free(buff);
		free(hwID);
		free(hwID2);
		free(fileDir);
		free(fileName);
		free(tBuff);
		return MAIN_REGISTER_FAIL;
	}
	free(tBuff);
	memset(buff, 0, 3);
	
	/* We also need to turn off the ability to have multiple packets in
	 * flight at once.  This makes it much easier to ensure proper message
	 * delivery.
	 */
	if(writeSerial("ATS06D=1:TG-ETRX1", &serial, strlen("ATS06D=1:TG-ETRX1"), TRUE))
	{
		fprintf(stderr, "Could not write to %s.\n", comPort);
		CloseHandle((PVOID)&serial);
		CloseHandle((PVOID)&log);
		free(comPort);
		free(buff);
		free(hwID);
		free(hwID2);
		free(fileDir);
		free(fileName);
		return MAIN_DATA_WRITE_ERROR;
	}
	
	strncpy(buff, "OK", 3);
	tBuff = getMessage(&serial, "OK", &log);
	if(strstr(tBuff, "ERROR:"))
	{
		fprintf(stderr, "Could not set register value 06D, %s.\n", tBuff);
		CloseHandle((PVOID)&serial);
		CloseHandle((PVOID)&log);
		free(comPort);
		free(buff);
		free(hwID);
		free(hwID2);
		free(fileDir);
		free(fileName);
		free(tBuff);
		return MAIN_REGISTER_FAIL;
	}
	free(tBuff);
	memset(buff, 0, 3);
	#endif /* TELEGESIS */
	
	/* Estabish a new PAN with this device as the coordinator.  We wait for
	 * three responses because the ETRX2USB device sends the response in
	 * three sections (and we'll time out between them).
	 */
	if(writeSerial("AT+EN", &serial, strlen("AT+EN"), TRUE))
	{
		fprintf(stderr, "Could not write to %s.\n", comPort);
		CloseHandle((PVOID)&serial);
		CloseHandle((PVOID)&log);
		free(comPort);
		free(buff);
		free(hwID);
		free(hwID2);
		free(fileDir);
		free(fileName);
		return MAIN_DATA_WRITE_ERROR;
	}
	
	strncpy(buff, "JPAN:", 6);
	tBuff = getMessage(&serial, "JPAN:", &log);
	if(!strstr(tBuff, "JPAN:"))
	{
		fprintf(stderr, "Could not establish PAN.\n");
		CloseHandle((PVOID)&serial);
		CloseHandle((PVOID)&log);
		free(comPort);
		free(buff);
		free(hwID);
		free(hwID2);
		free(fileDir);
		free(fileName);
		free(tBuff);
		return MAIN_JOIN_FAIL;
	}
	memset(buff, 0, 6);
	strncpy(buff, tBuff+5, 7);
	fprintf(stdout, "Established network %s on channel %c%c.\n",
		buff+3, *buff, *(buff+1));
	free(tBuff);
	memset(buff, 0, 150);

	/* Now, we need to make sure that our buffers are completely clear. */
	PurgeComm(&serial, PURGE_RXCLEAR);
	
	/* From now on, we're only concerned with receiving images.  The software
	 * on the ETRX2USB device should be handling any new nodes, routing
	 * things, etc.
	 */
	while(1)
	{
		/* Wait for data to arrive. */
		SetCommMask(serial, EV_RXCHAR);
		WaitCommEvent(serial, &eventMask, NULL);
		tBuff = NULL;
		
		/* Make sure our buffers are clear. */
		memset(buff, 0, 150);
		memset(hwID, 0, 17);
		
		/* Get the message and write it to the log file. */
		i = readMessage(&serial, buff);
		WriteFile(log, (LPVOID)buff, i, &bWritten, NULL);

		/* Format the message so we can use it. If it doesn't have at
		 * least 16 characters, it doesn't have a hardware ID and thus
		 * we don't want to actually do anything with it.
		 */
		if(i >= 16)
		{
			tBuff = (LPSTR)calloc(150, sizeof(CHAR));
			formatMessage(tBuff, buff, hwID, 0, 150, 150, 17);
		}
		else
		{
			continue;
		}
		
		/* Now, we need to see what the message was.  If a picture is ready,
		 * then we need to act on that signal.  Otherwise, we can just ignore
		 * the message (we already wrote it to the log file).
		 */
		if(strncmp(tBuff, "IMAGE_READY", 11) == 0)
		{
			/* Determine how many packets the image will be.  This number
			 * should be a four-digit hex value in the form 0xNNNN where
			 * N ranges from 0 to F.  Since we haven't received any packets
			 * yet, recPackets must be 0.
			 */
			numPackets = strtol(tBuff+11, NULL, 16);
			recPackets = 0x0;
			free(tBuff);
			
			/* Store our hardware ID for future reference. */
			memset(hwID2, 0, 17);
			strcpy(hwID2, hwID);
			
			/* Before we send the OK to the sender, we must first create
			 * the new image file.  To do so, we must obtain the current
			 * time.  Then, we compare it with the last time that we made
			 * an image.  If they match, we increment filesMade.  Else, we
			 * set filesMade to 0 because we haven't yet made any files with
			 * this timestamp.
			 * 
			 * In both cases, we then set lastTime equal to currentTime for
			 * future comparisons.
			 */
			GetLocalTime((LPSYSTEMTIME)&currTime);
			if(compareTimeStructs(&currTime, &lastTime) == 0)
				++filesMade;
			else
				filesMade = 0;
			setEqualTimeStructs(&lastTime, &currTime);
				
			/* Next, we determine the filename for the new file and then
			 * create it.  Filenames are of the format:
			 *	ddmmyyyy_hhmmss_x.jpg
			 * where dd is the two-digit day of the month, mm is the two-
			 * digit month, yyyy is a four-digit representation of the
			 * current year, hh, mm, and ss are the current hour, minute,
			 * and second values (each two-digits), and x is filesMade.
			 * We must always remember to prefix the filename with the
			 * user-specified directory.
			 */
			memset(fileName, 0, 1024);
			sprintf(fileName, "%s%.2d%.2d%.2d_%.2d%.2d%.2d_%.2d.jpg", fileDir,
				currTime.wDay, currTime.wMonth, currTime.wYear,
				currTime.wHour, currTime.wMinute, currTime.wSecond,
				filesMade);
			imageFile = CreateFile((LPCSTR)fileName, GENERIC_WRITE|GENERIC_READ,
				FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, CREATE_ALWAYS,
				FILE_FLAG_SEQUENTIAL_SCAN, NULL);
			if(imageFile == INVALID_HANDLE_VALUE)
			{
				fprintf(stderr, "Could not create new image files: %s.\n",
					fileName);
				CloseHandle((PVOID)&serial);
				CloseHandle((PVOID)&log);
				free(comPort);
				free(buff);
				free(hwID);
				free(hwID2);
				free(fileDir);
				free(fileName);
				return MAIN_OUTPUT_FILE_FAIL;
			}
						
			/* Now, send the sender a message that says it's okay to begin
			 * sending us data.  This message must be of the form:
			 *	IMAGE_READY_OK
			 * We'll try this ten times, then quit if we don't get an ACK.
			 */
			memset(buff, 0, 150);
			sprintf(buff, "AT+UCAST:%s=%s", hwID2, "IMAGE_READY_OK");
			for(i = 0; i < 10; i++)
			{
				writeSerial(buff, &serial, strlen(buff), TRUE);
				fprintf(stdout, "\tSent IMAGE_READY_OK signal.\n");
				strncpy(hwID, "OK", 3);
				tBuff = getMessage(&serial, "OK", &log);
				memset(hwID, 0, 3);
				if(strstr(tBuff, "ERROR:"))
				{
					fprintf(stdout, "\tReceived NACK from %s.\n", hwID2);
					free(tBuff);
					if(i == 9)
					{
						++i;
						break;
					}
				}
				else
				{
					fprintf(stdout, "\tGot ACK from %s.\n", hwID2);
					free(tBuff);
					break;
				}
			}
			
			if(i == 10)
			{
				CloseHandle((PVOID)&imageFile);
				fprintf(stdout, "\r\n*****No response from %s, time out.\r\n",
					hwID2);
				continue;
			}
			
			/* It's possible that a node will be disconnected from the
			 * network after sending us the IMAGE_READY signal and before
			 * sending us all of the image data.  To prevent the system
			 * from blocking forever, we establish a timer.  If we don't
			 * receive the first packet within 10 seconds of the IMAGE_READY
			 * signal, we consider the transmission void and write an error
			 * to the log file.  Each time we receive a packet, we reset the
			 * timer.  This effectively gives us a 10s timeout on each
			 * transmission.
			 */
			GetLocalTime((LPSYSTEMTIME)&currTime);
			
			/* We consider this the start of the image transfer. */
			tSec = currTime.wMilliseconds + (currTime.wSecond * 1000) + (currTime.wMinute * 60000);
			
			/* Now, we need to get the image data.  We'll get exactly the
			 * number of packets specified in the IMAGE_READY signal.  Each
			 * packet can contain 150 bytes of data, though the ETRX2USB 
			 * actually limits them to 65 bytes of data.
			 */
			while(recPackets < numPackets)
			{
				/* Get the current time and see if it has been 10 seconds
				 * since we last received a packet.  If so, write the error
				 * to the log file and break.
				 */
				GetLocalTime((LPSYSTEMTIME)&timer);
				if((timer.wSecond >= currTime.wSecond-50 
						&& timer.wMinute > currTime.wMinute)
					|| ((timer.wSecond)-10 >= currTime.wSecond 
						&& timer.wMinute == currTime.wMinute))
				{
					memset(buff, 0, 150);
					sprintf(buff, "\r\n*****TIMEOUT: %s (%s)\r\n",
						hwID2, fileName);
					WriteFile(log, (LPVOID)buff, strlen(buff), &bWritten, NULL);
					fprintf(stdout, "\tImage %s from %s timed out!\n",
						fileName, hwID2);
					break;
				}
				
				/* If a byte has arrived, we must read the message, format it,
				 * and write the useful part to the image file.  We then reset
				 * the timer, increment the number of packets received, and
				 * continue.
				 */
				memset(buff, 0, 150);
				if((i = readMessage(&serial, buff)) > 0)
				{
					i = formatMessage(tBuff, buff, hwID, 1, 150, 150, 17);
					fprintf(stdout, "Got packet %li of %li (%i bytes).\n",
						recPackets+1, numPackets, i);
						
					if(strcmp(hwID, hwID2) != 0)
					{
						fprintf(stdout, "Got packet from %s, expecting %s.\n",
							hwID, hwID2);
						continue;
					}

					WriteFile(imageFile, (LPVOID)tBuff, i, &bWritten, NULL);
					GetLocalTime((LPSYSTEMTIME)&currTime);
					++recPackets;
					continue;
				}
			}
			
			/* We stop our timer now, since we've finished the image transfer. */
			GetLocalTime((LPSYSTEMTIME)&currTime);
			tSec = ((currTime.wMilliseconds + (currTime.wSecond * 1000) +
						(currTime.wMinute * 60000)) - tSec);
			
			/* We're finished with this file, so we close the handle, write
			 * a success message, and return to the beginning of the loop.
			 */
			CloseHandle((PVOID)&imageFile);
			/*memset(tmpName, 0, 1024);
			sprintf(tmpName, "%s.jpg", fileName);
			MoveFile(fileName, tmpName);*/
			fprintf(stdout, "Finished writing image (%li of %li packets)",
				recPackets, numPackets);
			fprintf(stdout, ", %li milliseconds.\n", tSec);
			WriteFile(log, "\r\nFinished writing image.\r\n", 
				strlen("\r\nFinished writing image.\r\n"), &bWritten, NULL);
			continue;
		}
		else
		{
			free(tBuff);
			continue;
		}
	}

	/* Close the files. */
	CloseHandle((PVOID)&log);
	CloseHandle((PVOID)&serial);
	
	/* Free any used memory and exit. */
	free(comPort);
	free(buff);
	free(hwID);
	free(hwID2);
	free(fileDir);
	free(fileName);

	return MAIN_SUCCESS;
}
